package cn.com.qmth.examcloud.core.basic.service.impl;

import static org.springframework.data.domain.ExampleMatcher.GenericPropertyMatchers.contains;

import java.io.InputStream;
import java.util.ArrayList;
import java.util.List;

import javax.persistence.criteria.Predicate;

import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Example;
import org.springframework.data.domain.ExampleMatcher;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.google.common.collect.Lists;

import cn.com.qmth.examcloud.commons.base.exception.StatusException;
import cn.com.qmth.examcloud.commons.base.util.excel.ExcelError;
import cn.com.qmth.examcloud.commons.base.util.excel.ExcelReader;
import cn.com.qmth.examcloud.commons.base.util.excel.ExcelReaderHandle;
import cn.com.qmth.examcloud.commons.web.security.enums.RoleMeta;
import cn.com.qmth.examcloud.core.basic.dao.ExamSiteRepo;
import cn.com.qmth.examcloud.core.basic.dao.OrgRepo;
import cn.com.qmth.examcloud.core.basic.dao.RoleRepo;
import cn.com.qmth.examcloud.core.basic.dao.UserRepo;
import cn.com.qmth.examcloud.core.basic.dao.UserRoleRelationRepo;
import cn.com.qmth.examcloud.core.basic.dao.constants.Consts;
import cn.com.qmth.examcloud.core.basic.dao.entity.ExamSite;
import cn.com.qmth.examcloud.core.basic.dao.entity.Org;
import cn.com.qmth.examcloud.core.basic.dao.entity.RoleEntity;
import cn.com.qmth.examcloud.core.basic.dao.entity.UserEntity;
import cn.com.qmth.examcloud.core.basic.dao.entity.UserRoleRelationEntity;
import cn.com.qmth.examcloud.core.basic.dao.enums.OrgType;
import cn.com.qmth.examcloud.core.basic.service.bean.OrgDto;

@Service
public class OrgService {

	@Autowired
	RoleRepo roleRepo;

	@Autowired
	OrgRepo orgRepo;

	@Autowired
	UserRepo userRepo;

	@Autowired
	ExamSiteRepo examSiteRepo;

	@Autowired
	UserRoleRelationRepo userRoleRelationRepo;

	@Transactional
	public List<ExcelError> importLearnCenter(Long orgId, InputStream inputStream) {
		ExcelReader excelReader = new ExcelReader(OrgDto.class);
		List<ExcelError> excelErrors = excelReader.reader(inputStream, new ExcelReaderHandle() {
			@Override
			public ExcelError handle(Object obj) {
				OrgDto dto = (OrgDto) obj;
				dto.setParentId(orgId);
				dto.setRootId(orgId);
				ExcelError error = importCheck(dto);
				if (error == null) {
					saveOrgAndExamSite(dto);
				}
				return error;
			}
		});
		return excelErrors;
	}

	/**
	 * 保存学习中心和考点
	 * 
	 * @param orgDto
	 */
	private void saveOrgAndExamSite(OrgDto orgDto) {
		Org org = orgRepo.findByParentIdAndCode(orgDto.getParentId(), orgDto.getCode());
		if (org == null) {
			Org tempOrg = orgRepo.save(orgAssembler(orgDto));
			orgDto.setId(tempOrg.getId());
			examSiteRepo.save(examSiteAssembler(orgDto));
			createLearnerCenterUser(tempOrg);
		} else {
			org.setName(orgDto.getName());
			org.setContacts(orgDto.getContacts());
			org.setTelphone(orgDto.getTelphone());
			orgRepo.save(org);

			orgDto.setId(org.getId());
			ExamSite examSite = examSiteRepo.findFirstByOrgIdAndCode(org.getId(),
					orgDto.getExamSiteCode());
			if (examSite == null) {
				examSiteRepo.save(examSiteAssembler(orgDto));
			} else {
				examSite.setName(orgDto.getExamSiteName());
				examSiteRepo.save(examSite);
			}
		}

	}

	/**
	 * 创建学习中心用户
	 * 
	 * @param org
	 */
	public void createLearnerCenterUser(Org org) {
		UserEntity user = userRepo.findByLoginName(org.getCode());
		if (null != user) {
			return;
		}
		user = new UserEntity();
		user.setRootOrgId(org.getRootId());
		user.setOrgId(org.getId());
		user.setLoginName(org.getCode());
		user.setName(org.getContacts());
		user.setMobile(org.getTelphone());
		user.setEnable(true);
		user.setPassword(Consts.DEFAULT_PASSWORD);
		UserEntity saved = userRepo.save(user);
		List<UserRoleRelationEntity> userRoles = Lists.newArrayList();
		RoleEntity role = roleRepo.findByCode(RoleMeta.LC_USER.name());
		UserRoleRelationEntity relation = new UserRoleRelationEntity(saved.getId(), role.getId(),
				role.getCode());
		userRoles.add(relation);
		userRoleRelationRepo.save(userRoles);
	}

	private Org orgAssembler(OrgDto orgDto) {
		Org org = new Org();
		org.setRootId(orgDto.getRootId());
		org.setParentId(orgDto.getParentId());
		org.setCode(orgDto.getCode());
		org.setName(orgDto.getName());
		org.setContacts(orgDto.getContacts());
		org.setTelphone(orgDto.getTelphone());
		org.setEnable(true);
		// 添加机构类型
		org.setType(OrgType.SCHOOL);
		return org;
	}

	private ExamSite examSiteAssembler(OrgDto orgDto) {
		ExamSite examSite = new ExamSite();
		examSite.setCode(orgDto.getExamSiteCode());
		examSite.setName(orgDto.getExamSiteName());
		examSite.setOrgId(orgDto.getId());
		examSite.setEnable(true);
		return examSite;
	}

	private ExcelError importCheck(OrgDto dto) {
		if (StringUtils.isBlank(dto.getCode())) {
			return new ExcelError("中心代码不能为空");
		}
		if (StringUtils.isBlank(dto.getName())) {
			return new ExcelError("中心名称不能为空");
		}
		if (StringUtils.isBlank(dto.getExamSiteCode())) {
			return new ExcelError("考点代码不能为空");
		}
		if (StringUtils.isBlank(dto.getExamSiteName())) {
			return new ExcelError("考点名称不能为空");
		}
		if (StringUtils.isBlank(dto.getContacts())) {
			return new ExcelError("联系人不能为空");
		}
		if (StringUtils.isBlank(dto.getTelphone())) {
			return new ExcelError("联系电话不能为空");
		}
		return null;
	}

	/**
	 * 启用/禁用机构；禁用机构时禁用该机构下的所有用户
	 * 
	 * @param id
	 * @param enable
	 * @return
	 */
	@Transactional
	public Org enableSchool(Long id, boolean enable) {
		Org org = orgRepo.findOne(id);
		org.setEnable(enable);
		orgRepo.save(org);
		return org;
	}

	public Page<Org> findAll(Org orgCriteria, Pageable pageable) {
		if (orgCriteria.getType() == null) {
			Specification<Org> specification = getSpecification(orgCriteria);
			return orgRepo.findAll(specification, pageable);
		}
		ExampleMatcher exampleMatcher = ExampleMatcher.matching().withMatcher("name", contains())
				.withMatcher("code", contains());
		Example<Org> examExamStudentple = Example.of(orgCriteria, exampleMatcher);
		return orgRepo.findAll(examExamStudentple, pageable);
	}

	private Specification<Org> getSpecification(Org orgCriteria) {
		Specification<Org> specification = (root, query, cb) -> {
			List<Predicate> predicates = new ArrayList<>();
			if (!StringUtils.isEmpty(orgCriteria.getName())) {
				predicates.add(cb.equal(root.get("name"), orgCriteria.getName()));
			}
			if (!StringUtils.isEmpty(orgCriteria.getCode())) {
				predicates.add(cb.equal(root.get("code"), orgCriteria.getCode()));
			}
			predicates.add(cb.notEqual(root.get("type"), OrgType.SCHOOL));
			predicates.add(cb.equal(root.get("parentId"), orgCriteria.getParentId()));
			return cb.and(predicates.toArray(new Predicate[predicates.size()]));
		};
		return specification;
	}

	public List<Org> findAll(Org orgCriteria) {
		ExampleMatcher exampleMatcher = ExampleMatcher.matching().withMatcher("name", contains())
				.withMatcher("code", contains());
		Example<Org> examExample = Example.of(orgCriteria, exampleMatcher);
		return orgRepo.findAll(examExample);
	}

	/**
	 * 强校验
	 *
	 * @author WANGWEI
	 * @param org
	 * @return
	 */
	public Org save(Org org) {

		String code = org.getCode();
		if (StringUtils.isBlank(code)) {
			throw new StatusException("B-150000", "code is blank");
		}

		// 顶级机构
		if (null == org.getParentId()) {

			// 校验顶级机构编码已存在
			Org rootOrg = orgRepo.findRootOrg(code);
			if (null != rootOrg) {
				if (!(null != org.getId() && org.getId().equals(rootOrg.getId()))) {
					throw new StatusException("B-150001", "顶级机构编码已存在");
				}
			}

			// 新增
			if (null == org.getId()) {
				org.setRootId(-1L);
				Org saved = orgRepo.save(org);
				org.setId(saved.getId());
				org.setRootId(saved.getId());
			}
		}
		// 子机构
		else {
			Long rootId = org.getRootId();
			if (null == rootId) {
				throw new StatusException("B-150002", "rootId is null");
			}
			Org rootOrg = orgRepo.findOne(rootId);
			if (null == rootOrg || null != rootOrg.getParentId()) {
				throw new StatusException("B-150003", "rootId is wrong");
			}
			Long parentId = org.getParentId();

			Org parentOrg = orgRepo.findOne(parentId);
			if (!parentOrg.getRootId().equals(rootId)) {
				throw new StatusException("B-150004", "parentId is wrong");
			}

			Org subOrg = orgRepo.findByRootIdAndCode(rootId, code);
			if (null != subOrg) {
				if (!(null != org.getId() && org.getId().equals(subOrg.getId()))) {
					throw new StatusException("B-150005", "机构编码已存在");
				}
			}

		}

		Org saved = orgRepo.save(org);
		return saved;
	}

	public Org findOne(Long id) {
		Org org = orgRepo.findOne(id);
		return org;
	}

}
